@@ -170,7 +170,8 @@ a_crese_mail_followup_to(struct message *mp){
       l = strlen(tr) + strlen(mft->n_name) + 3 +1;
       sp = n_lofi_alloc(l);
 
-      snprintf(sp, l, tr, mft->n_name, (mft->n_flink != NULL ? "..." : n_empty));
+      snprintf(sp, l, tr, mft->n_name,
+         (mft->n_flink != NULL ? "..." : n_empty));
       if(quadify(cp, UIZ_MAX, sp, TRU1) <= FAL0)
          mft = NULL;
 
@@ -251,8 +252,9 @@ jlink:
          hp->h_to = NULL;
          once = TRU1;
          goto jredo;
-      }
-      hp->h_to = np;
+      }else
+         for(hp->h_to = np; np != NULL; np = np->n_flink)
+            np->n_type = (np->n_type & ~GMASK) | GTO;
    }
    NYD2_LEAVE;
 }
