@@ -45,7 +45,7 @@
  * was large enough to swallow mailname */
 static bool_t  _update_mailname(char const *name);
 #ifdef HAVE_C90AMEND1 /* TODO unite __narrow_suffix() into one fun! */
-SINLINE size_t __narrow_suffix(char const *cp, size_t cpl, size_t maxl);
+n_INLINE size_t __narrow_suffix(char const *cp, size_t cpl, size_t maxl);
 #endif
 
 /**/
@@ -128,7 +128,7 @@ jdocopy:
 }
 
 #ifdef HAVE_C90AMEND1
-SINLINE size_t
+n_INLINE size_t
 __narrow_suffix(char const *cp, size_t cpl, size_t maxl)
 {
    int err;
