@@ -1,12 +1,12 @@
 /*@ S-nail - a mail user agent derived from Berkeley Mail.
- *@ Privilege-separated dot file lock program (WANT_DOTLOCK=yes)
+ *@ Privilege-separated dot file lock program (OPT_DOTLOCK=yes)
  *@ that is capable of calling setuid(2) and change its user identity
- *@ to the configured PRIVSEP_USER (usually "root"), in order to create
+ *@ to the configured VAL_PRIVSEP_USER (usually "root"), in order to create
  *@ a dotlock file with the same UID/GID as the mailbox to be locked.
  *@ It should be started when chdir(2)d to the lock file's directory,
- *@ and SIGPIPE should be ignored.
+ *@ with a symlink-resolved target and with SIGPIPE being ignored.
  *
- * Copyright (c) 2015 Steffen (Daode) Nurpmeso <sdaoden@users.sf.net>.
+ * Copyright (c) 2015 - 2017 Steffen (Daode) Nurpmeso <steffen@sdaoden.eu>.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -26,13 +26,13 @@
 
 #include "nail.h"
 
+static void _ign_signal(int signum);
+static uiz_t n_msleep(uiz_t millis, bool_t ignint);
+
 #include "dotlock.h"
 
-static void _ign_signal(int signum);
-
 static void
-_ign_signal(int signum)
-{
+_ign_signal(int signum){
    struct sigaction nact, oact;
 
    nact.sa_handler = SIG_IGN;
@@ -41,37 +41,65 @@ _ign_signal(int signum)
    sigaction(signum, &nact, &oact);
 }
 
+static uiz_t
+n_msleep(uiz_t millis, bool_t ignint){
+   uiz_t rv;
+
+#ifdef HAVE_NANOSLEEP
+   /* C99 */{
+      struct timespec ts, trem;
+      int i;
+
+      ts.tv_sec = millis / 1000;
+      ts.tv_nsec = (millis %= 1000) * 1000 * 1000;
+
+      while((i = nanosleep(&ts, &trem)) != 0 && ignint)
+         ts = trem;
+      rv = (i == 0) ? 0 : (trem.tv_sec * 1000) + (trem.tv_nsec / (1000 * 1000));
+   }
+
+#elif defined HAVE_SLEEP
+   if((millis /= 1000) == 0)
+      millis = 1;
+   while((rv = sleep((unsigned int)millis)) != 0 && ignint)
+      millis = rv;
+#else
+# error Configuration should have detected a function for sleeping.
+#endif
+   return rv;
+}
+
 int
-main(int argc, char **argv)
-{
+main(int argc, char **argv){
    char hostbuf[64];
-   struct dotlock_info di;
+   struct n_dotlock_info di;
    struct stat stb;
    sigset_t nset, oset;
-   enum dotlock_state dls;
+   enum n_dotlock_state dls;
 
    /* We're a dumb helper, ensure as much as we can noone else uses us */
-   if (argc != 12 ||
-         strcmp(argv[ 0], PRIVSEP) ||
+   if(argc != 12 ||
+         strcmp(argv[ 0], VAL_PRIVSEP) ||
          (argv[1][0] != 'r' && argv[1][0] != 'w') ||
          strcmp(argv[ 1] + 1, "dotlock") ||
          strcmp(argv[ 2], "mailbox") ||
+         strchr(argv[ 3], '/') != NULL /* Seal path injection.. */ ||
          strcmp(argv[ 4], "name") ||
          strcmp(argv[ 6], "hostname") ||
          strcmp(argv[ 8], "randstr") ||
-         strchr(argv[ 9], '/') != NULL /* Seal path injection vector */ ||
+         strchr(argv[ 9], '/') != NULL /* ..attack vector */ ||
          strcmp(argv[10], "pollmsecs") ||
          fstat(STDIN_FILENO, &stb) == -1 || !S_ISFIFO(stb.st_mode) ||
-         fstat(STDOUT_FILENO, &stb) == -1 || !S_ISFIFO(stb.st_mode)) {
+         fstat(STDOUT_FILENO, &stb) == -1 || !S_ISFIFO(stb.st_mode)){
 jeuse:
       fprintf(stderr,
-         "This is a helper program of \"" UAGENT "\" (in " BINDIR ").\n"
-         "  It is capable of gaining more privileges than \"" UAGENT "\"\n"
+         "This is a helper program of " VAL_UAGENT " (in " VAL_BINDIR ").\n"
+         "  It is capable of gaining more privileges than " VAL_UAGENT "\n"
          "  and will be used to create lock files.\n"
-         "  It's sole purpose is outsourcing of high privileges into\n"
+         "  The sole purpose is outsourcing of high privileges into\n"
          "  fewest lines of code in order to reduce attack surface.\n"
-         "  It cannot be run by itself.\n");
-      exit(EXIT_USE);
+         "  This program cannot be run by itself.\n");
+      exit(n_EXIT_USE);
    }else{
       /* Prevent one more path injection attack vector, but be friendly */
       char const *ccp;
@@ -94,10 +122,12 @@ jeuse:
    di.di_hostname = argv[7];
    di.di_randstr = argv[9];
    di.di_pollmsecs = (size_t)strtoul(argv[11], NULL, 10);
-   {
+
+   /* Ensure the lock name and the file name are identical */
+   /* C99 */{
       size_t i = strlen(di.di_file_name);
 
-      if (i == 0 || strncmp(di.di_file_name, di.di_lock_name, i) ||
+      if(i == 0 || strncmp(di.di_file_name, di.di_lock_name, i) ||
             di.di_lock_name[i] == '\0' || strcmp(di.di_lock_name + i, ".lock"))
          goto jeuse;
    }
@@ -117,25 +147,25 @@ jeuse:
    sigdelset(&nset, SIGCONT); /* (Rather redundant, though) */
    sigprocmask(SIG_BLOCK, &nset, &oset);
 
-   dls = DLS_NOPERM | DLS_ABANDON;
+   dls = n_DLS_NOPERM | n_DLS_ABANDON;
 
    /* First of all: we only dotlock when the executing user has the necessary
     * rights to access the mailbox */
-   if (access(di.di_file_name, (argv[1][0] == 'r' ? R_OK : R_OK | W_OK)))
+   if(access(di.di_file_name, (argv[1][0] == 'r' ? R_OK : R_OK | W_OK)))
       goto jmsg;
 
    /* We need UID and GID information about the mailbox to lock */
-   if (stat(di.di_file_name, di.di_stb = &stb) == -1)
+   if(stat(di.di_file_name, di.di_stb = &stb) == -1)
       goto jmsg;
 
-   dls = DLS_PRIVFAILED | DLS_ABANDON;
+   dls = n_DLS_PRIVFAILED | n_DLS_ABANDON;
 
    /* This privsep helper only gets executed when needed, it thus doesn't make
     * sense to try to continue with initial privileges */
-   if (setuid(geteuid()))
+   if(setuid(geteuid()))
       goto jmsg;
 
-   dls = _dotlock_create(&di);
+   dls = a_dotlock_create(&di);
 
    /* Finally: notify our parent about the actual lock state.. */
 jmsg:
@@ -144,14 +174,14 @@ jmsg:
 
    /* ..then eventually wait until we shall remove the lock again, which will
     * be notified via the read returning */
-   if (dls == DLS_NONE) {
+   if(dls == n_DLS_NONE){
       read(STDIN_FILENO, &dls, sizeof dls);
 
       unlink(di.di_lock_name);
    }
 
    sigprocmask(SIG_SETMASK, &oset, NULL);
-   return (dls == DLS_NONE ? EXIT_OK : EXIT_ERR);
+   return (dls == n_DLS_NONE ? n_EXIT_OK : n_EXIT_ERR);
 }
 
 /* s-it-mode */
